<?php

class Media_Rename {

	private $is_media_rename_page;
	private $current_path;

	function __construct() {
		$post=get_post($_GET['post']);
		$is_media_edit_page = $post && $post->post_type=='attachment' && $GLOBALS['pagenow']=='post.php';
		$is_media_listing_page = $GLOBALS['pagenow']=='upload.php';
		$this->is_media_rename_page = $is_media_edit_page || $is_media_listing_page;
		$this->current_path=add_query_arg();
	}

	// Adds the "New Filename" column at the media posts listing page
	function add_new_filename_column($columns) {  
		$columns['new_filename'] = 'New Filename';
		return $columns;
	}


	// Adds the "New Filename" column content at the media posts listing page
	function add_new_filename_column_content($column_name, $post_id) {
		$file_parts=$this->get_file_parts($post_id);
	    echo $this->get_new_filename_field($post_id, $file_parts[0], $file_parts[1]);
	}


	// Add the "New Filename" field to the Media form
	function add_filename_field($form_fields, $post) {
		if ($GLOBALS['post']->post_type=='attachment') {
			$file_parts=$this->get_file_parts($GLOBALS['post']->ID);
			$form_fields['mr_new_filename']=array(
				'label' => __('New Filename'),
				'input' => 'html',
				'html' => $this->get_new_filename_field($GLOBALS['post']->ID, $file_parts[0], $file_parts[1])
			);
		}
		return $form_fields;
	}


	// Makes sure that the success message will be shown on bulk rename
	function handle_bulk_rename_form_submit() {
		if ($_GET['action'] == 'rename') {
			wp_redirect( add_query_arg(array('renamed' => 1), wp_get_referer()) );
			exit;
		} elseif ($_GET['renamed'] && count($_GET)>1) {
			wp_redirect( remove_query_arg('renamed', $this->current_path) );
			exit;
		}
	}


	// Shows bulk rename success notice
	function show_bulk_rename_success_notice() {
		if($_REQUEST['renamed']) {
			echo '<div class="updated"><p>Medias successfully renamed!</p></div>';
		}
	}


	// Print the JS code only on media.php and media-upload.php pages
	function print_js() {
		if ($this->is_media_rename_page) { ?>
			<script type="text/javascript">
				mr_renameText = "<?php _e('Rename') ?>";
				mr_unknownError = "<?php _e('Unknown error!') ?>";
			</script>
			<?php wp_enqueue_script('media-rename', plugins_url('js/scripts.js', __FILE__), array('jquery'));
		}
	}


	// Print the CSS styles only on media.php and media-upload.php pages
	function print_css() {
		if ($this->is_media_rename_page) {
			wp_register_style('media-rename', plugins_url('css/style.css', __FILE__));
			wp_enqueue_style('media-rename');
		}
	}


	// Prints the "New Filename" textfield
	function get_new_filename_field($post_id, $filename, $extension) {
		if (!isset($GLOBALS['mr_nonce_printed'])) $GLOBALS['mr_nonce_printed']=0;
		ob_start(); ?>
			<div class="mr_field_wrapper">
				<input type="text" class="text" value="<?php echo $filename ?>" title="<?php echo $filename ?>" data-post-id="<?php echo $post_id ?>" />
				<span class="mr_file_ext">.<?php echo $extension ?></span>
				<span class="mr_loader"></span>
				<span class="mr_success"></span>
				<span class="mr_error"></span>
				<?php if (!$GLOBALS['mr_nonce_printed']) {
					wp_nonce_field('media_rename', '_mr_wp_nonce');
					$GLOBALS['mr_nonce_printed']++;
				} ?>
			</div>
		<?php return ob_get_clean();
	}


	// 
	function ajax_rename() {
		if (check_ajax_referer('media_rename', '_wpnonce', 0)) {
			echo $this->do_rename($_POST['post_id'], $_POST['new_filename']);
		}
		exit;
	}


	// Handles the actual rename process
	static function do_rename($post_id, $new_filename) {
		// Variables
		$post=get_post($post_id); // the attachment post object

		//Validations
		if (!$post) return __('Post with ID ' . $post_id . ' does not exist!');
		if ($post && $post->post_type!='attachment') return __('Post with ID ' . $post_id . ' is not an attachment!');
		if (!$new_filename) return __('The field is empty!');
		if ($new_filename != str_replace(array('%', '^'), '', sanitize_file_name($new_filename))) return __('Bad characters or invalid filename!');

		// Variables
		preg_match('~(wp-content.*)/~', get_attached_file($post->ID), $folder_rel_path); // relative path of the media containing folder
		$folder_rel_path=$folder_rel_path[1];
		$file_parts=self::get_file_parts($post_id);
		$file_rel_path=str_replace($file_parts[0], $new_filename, get_post_meta($post->ID, '_wp_attached_file', 1)); // relative path of the media file
		$attach_metadata=get_post_meta($post->ID, '_wp_attachment_metadata', 1); // get atachment metadata array
		$posts=get_posts('post_type=any&post_status=any&numberposts=-1');
		$post_content_searches=array(); // the strings (urls) to search for
		$post_content_replaces=array(); // the updated strings (urls) to replace the above with
		preg_match('~(.*)/~', $file_rel_path, $year_month_url_part);
		$wp_upload_dir=wp_upload_dir();
		$folder_abs_url=$wp_upload_dir['baseurl'] . '/' . $year_month_url_part[0];

		//Validations
		if (file_exists($wp_upload_dir['basedir'] . DIRECTORY_SEPARATOR . $file_rel_path)) return __('A file with that name already exists in the containing folder!');
		if (!is_writable(ABSPATH . $folder_rel_path)) return __('The media containing directory is not writable!');

		// Change the post
		$post_changes=array();
		$post_changes['ID']=$post->ID;
		$post_changes['post_name']=wp_unique_post_slug($new_filename, $post->ID, $post->post_status, $post->post_type, $post->post_parent);
		wp_update_post($post_changes);

		// Change post metas & rename files
		update_post_meta($post->ID, '_wp_attached_file', $file_rel_path); // change file relative path in post meta
		$attach_metadata['file']=$file_rel_path; // change the name of the default file
		@rename(ABSPATH . $folder_rel_path . DIRECTORY_SEPARATOR . $file_parts[0] . '.' . $file_parts[1], ABSPATH . $folder_rel_path . DIRECTORY_SEPARATOR . $new_filename . '.' . $file_parts[1]); // rename the default file
		$post_content_searches[]=$folder_abs_url . $file_parts[0] . '.' . $file_parts[1];
		$post_content_replaces[]=$folder_abs_url . $new_filename . '.' . $file_parts[1];
		if ($attach_metadata['sizes']) { // if is image and has size files
			foreach ($attach_metadata['sizes'] as &$size) {
				$new_size_filename=str_replace($file_parts[0], $new_filename, $size['file']);
				@rename(ABSPATH . $folder_rel_path . DIRECTORY_SEPARATOR . $size['file'], ABSPATH . $folder_rel_path . DIRECTORY_SEPARATOR . $new_size_filename); // rename current size file
				$post_content_searches[]=$folder_abs_url . $size['file'];
				$post_content_replaces[]=$folder_abs_url . $new_size_filename;
				$size['file']=$new_size_filename; // change filename for the current size
			}
		}
		update_post_meta($post->ID, '_wp_attachment_metadata', $attach_metadata); // apply default and size filenames by updating post meta

		// Replace the old with the new media link in the content of all posts
		foreach ($posts as $post) {
			$new_post=array(
				'ID'=>$post->ID,
				'post_content'=>$post->post_content
			);
			$new_post['post_content']=str_replace($post_content_searches, $post_content_replaces, $new_post['post_content']);
			wp_update_post($new_post);
		}

		return 1;
	}


	// Extracts filename and extension form an attachment post
	static function get_file_parts($post_id) {
		preg_match('~.*/(.*)\.(.*)$~', get_attached_file($post_id), $file_parts); // extract current filename and extension
		array_shift($file_parts);
		return $file_parts;
	}

}